<?php

namespace Modules\Candidatos\Http\Controllers\Admin;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\View;
use Modules\Candidatos\Entities\Candidato;
use Modules\Usuarios\Entities\Usuario;
use Modules\Empleos\Entities\EmpleoCandidato;
use Modules\Paginas\Entities\Servicio;
use Modules\Pruebas\Entities\PruebaCandidato;
use Modules\Pruebas\Entities\PruebaRespuesta;
use Modules\Pruebas\Entities\PruebaResultado;
use Modules\Candidatos\Http\Requests\Admin\CandidatoStoreRequest;
use Modules\Candidatos\Http\Requests\Admin\CandidatoUpdateRequest;
use DataTables;
use Illuminate\Support\Facades\Hash;

class CandidatosController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        return view('candidatos::admin.candidatos.index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('candidatos::admin.candidatos.create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(CandidatoStoreRequest $request)
    {
        $usuario = Usuario::create($request->except('_token','id','usuario_id','password_confirmation'));
        $usuario->assignRole('Candidato');
        session(['public.candidato.created' => true]);
        $candidato = $usuario->candidato()->create($request->only('nombre','apellido','email'));
        return redirect()
            ->route('dashboard.candidatos.show',$candidato->id)
            ->with('alert_success', 'El candidato ha sido creado.');
    } 

    /**
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show(Candidato $candidato)
    {
        return view('candidatos::admin.candidatos.show')->with(compact('candidato'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Response
     */
    public function edit(Candidato $candidato)
    {
        return view('candidatos::admin.candidatos.edit')->with(compact('candidato'));
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Response
     */
    public function update(CandidatoUpdateRequest $request, Candidato $candidato)
    {
        //dd($request->all());
        if (isset($request->password) ) {
            $data   = $request->only('nombre','apellido','email','activo','password');
            $data['password'] = Hash::make($data['password']);
        } else {
            $data   = $request->only('nombre','apellido','email','activo');
        }
        $candidato->update($request->only('nombre','apellido','email'));
        $candidato->usuario()->update($data);
        return redirect()
            ->route('dashboard.candidatos.show',$candidato->id)
            ->with('alert_success', 'Tu perfil ha sido actualizado correctamente.');
    } 

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Response
     */
    public function destroy(Candidato $candidato)
    {

        $candidato->entrevista()->delete();
        $candidato->estudios()->delete();
        $candidato->experiencia()->delete();
        $candidato->habilidades()->delete();
        $candidato->otros()->delete();
        $candidato->referencia()->delete();
        
        $empleo_candidato = EmpleoCandidato::where('candidato_id',$candidato->id)->get();
        
        foreach($empleo_candidato as $item){
            $servicios = Servicio::where('empleo_candidato_id',$item->id)->update(['empleo_candidato_id' => null]);
        }
        
        $pruebas_candidato = PruebaCandidato::where('candidato_id',$candidato->id)->get();

        foreach($pruebas_candidato as $item){
            $pruebas_respuestas = PruebaRespuesta::where('prueba_candidato_id',$item->id)->delete();
            $pruebas_resultado = PruebaResultado::where('prueba_candidato_id',$item->id)->delete();
        }

        $candidato->pruebas()->delete();
        //$candidato->empleos()->delete();
        $candidato->delete();
        $candidato->usuario()->delete(); 

        return redirect()->back()->with('alert_success', 'El candidato ha sido eliminado.');
    }


    public function candidatosList(Request $request){ 
        $usuario = Auth::user();
        $buscar = $request->get('search');
        $totalCandidatos = Candidato::get()->count();
        $candidatos = Candidato::selectRaw('candidatos.*')
        ->selectRaw("concat(candidatos.nombre, ' ', candidatos.apellido) as nombre")
        ->addSelect('usuarios.activo')
        ->addSelect('usuarios.logged_at')
        ->join('usuarios','candidatos.usuario_id','=','usuarios.id');
        if($buscar['value'] != ''){
            $candidatos = $candidatos->where('candidatos.email','like','%'.$buscar['value'].'%')
                ->orWhere('candidatos.id','like','%'.$buscar['value'].'%')
                ->orWhere('candidatos.nombre','like','%'.$buscar['value'].'%');
        }
        return DataTables::eloquent($candidatos)
        ->addColumn('action', function($candidato) use($usuario){
            $button = '<a class="btn btn-primary btn-sm" href="'.route('dashboard.candidatos.show',$candidato->id).'"><i class="fas fa-eye"></i></a>';
            if($usuario->hasPermissionTo('Editar Usuarios') || $usuario->hasRole('Super Admin')){
                $button .= '<a class="btn btn-info btn-sm" href="'.route('dashboard.candidatos.edit',$candidato->id).'"><i class="fas fa-pencil-alt"></i></a>';
            }
            if($usuario->hasPermissionTo('Eliminar Usuarios') || $usuario->hasRole('Super Admin')){
                $button .= '<a class="btn btn-danger btn-sm" href="#"   data-toggle="modal" data-target="#candidato-destroy-'.$candidato->id.'"><i class="fas fa-trash"></i></a>';
                $button .= view('candidatos::admin.candidatos.destroy',['candidato' => $candidato])->render(); 
            }
            return $button;
        })
        ->addColumn('estado', function($candidato){
            $html = ($candidato->activo) ? '<span class="badge badge-success">Si</span>':'<span class="badge badge-danger">No</span>';
            return $html;
        })
        ->addColumn('logged', function($candidato){
            $html = ($candidato->logged_at) ? $candidato->logged_at:'';
            return $html;
        })
        ->rawColumns(['action','estado','logged'])
        ->setTotalRecords($totalCandidatos)
        ->make(true);
    }
}
