<?php

namespace Modules\Empresas\Http\Controllers\Admin;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Yajra\DataTables\DataTables;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Modules\Empleos\Entities\Empleo;
use Modules\Empresas\Entities\Empresa;
use Modules\Usuarios\Entities\Usuario;
use Modules\Empresas\Emails\EmpresaHabilitada;
use Modules\Empresas\Http\Requests\Admin\EmpresaStoreRequest;
use Modules\Empresas\Http\Requests\Admin\EmpresaUpdateRequest;

class EmpresasController extends Controller
{
    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {
        return view('empresas::admin.empresas.index');
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {
        return view('empresas::admin.empresas.create');
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(EmpresaStoreRequest $request)
    {
        $usuario = Usuario::create($request->only('nombre','email','activo','password')+['apellido' => $request->nit]);
        $usuario->assignRole('Empresa');
        session(['public.empresa.created' => true]);
        $filename = '';
        $filename2 = '';
        if(isset($request->logotipo)){
            $extension = $request->logotipo->extension();
            $filename = sha1(time().time()).".$extension";
            $request->logotipo->move(public_path('storage/uploads/images'),$filename);
        }
        if(isset($request->rtu)){
            $extension = $request->rtu->extension();
            $filename2 = sha1(time().time()).".$extension";
            $request->rtu->move(public_path('storage/uploads/docs'),$filename2);
        }

        $fecha_servicio = $request->informe != 0 ? Carbon::now() : null;

        $empresa = $usuario->empresa()->create($request->except('_method','id','usuario_id','_token','password','password_confirmation','activo','logotipo','rtu', 'fecha_habilitacion_servicio')
        +['logotipo' => $filename, 'rtu' => $filename2, 'fecha_habilitacion_servicio' => $fecha_servicio]);


        return redirect()
            ->route('dashboard.empresas.show',$empresa->id)
            ->with('alert_success', 'El empresa ha sido creada.');
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show(Empresa $empresa)
    {
        $vacantes_aplicadas = Empleo::select('empleos.*')
        ->addSelect('empleo_candidatos.empleo_id as empleo_candidatos')
        ->join('empleo_candidatos','empleos.id','empleo_candidatos.empleo_id')
        ->whereIn('empleo_candidatos.empleo_id',( $empresa->empleos()->get('id') ))->count();

        return view('empresas::admin.empresas.show')->with(compact('empresa','vacantes_aplicadas'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Response
     */
    public function edit(Empresa $empresa)
    {
        return view('empresas::admin.empresas.edit')->with(compact('empresa'));
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Response
     */
    public function update(EmpresaUpdateRequest $request, Empresa $empresa)
    {
        if (isset($request->password) ) {
            $data   = $request->only('nombre','email','activo','password');
            $data['password'] = Hash::make($data['password']);
        } else {
            $data   = $request->only('nombre','email','activo');
        }
        if(isset($request->logotipo)){
            $extension = $request->logotipo->extension();
            $filename = sha1(time().time()).".$extension";
            $request->logotipo->move(public_path('storage/uploads/images'),$filename);
            $empresa->update(['logotipo' => $filename]);
        }if(isset($request->rtu)){
            $extension = $request->rtu->extension();
            $filename = sha1(time().time()).".$extension";
            $request->rtu->move(public_path('storage/uploads/docs'),$filename);
            $empresa->update(['rtu' => $filename]);
        }

        if ( ($empresa->usuario->activo == false) && ($request->activo == 1) ) {
            $data   = $request->only('activo');
            Mail::to($empresa->email)->send(new EmpresaHabilitada($empresa)); 
        } 

        $fecha_servicio = $request->informe != 0 ? Carbon::now() : null;

        $empresa->update($request->except('_method','id','usuario_id','_token','password','password_confirmation','activo','logotipo','rtu','fecha_habilitacion_servicio')+['fecha_habilitacion_servicio' => $fecha_servicio]);
        $empresa->usuario()->update($data+['apellido' => $request->nit]);
        return redirect()
            ->route('dashboard.empresas.show',$empresa->id)
            ->with('alert_success', 'El empresa ha sido actualizada.');
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Response
     */
    public function destroy(Empresa $empresa)
    {
        $empresa->delete();
        $empresa->usuario()->delete(); 

        return redirect()->back()->with('alert_success', 'La empresa ha sido eliminado.');
    }

    public function empresasList(Request $request){
        $usuario = Auth::user();
        $empresas = Empresa::select("empresas.nombre")
        ->addSelect('empresas.email')
        ->addSelect('empresas.id')
        ->addSelect('usuarios.activo')
        ->addSelect('usuarios.logged_at')
        ->addSelect('empresas.informe')
        ->addSelect('empresas.fecha_habilitacion_servicio')
        ->addSelect(DB::raw('DATE_FORMAT(empresas.created_at, "%d/%m/%Y") as fecha_creacion'))
        ->join('usuarios','empresas.usuario_id','=','usuarios.id');
        $empresas = $empresas->get();
        return DataTables::of($empresas)
        ->addColumn('action', function($empresa) use($usuario){
            $button = '<a class="btn btn-primary btn-sm" href="'.route('dashboard.empresas.show',$empresa->id).'"><i class="fas fa-eye"></i></a>';
            if($usuario->hasPermissionTo('Editar Empresas') || $usuario->hasRole('Super Admin|Admin')){
                $button .= '<a class="btn btn-info btn-sm" href="'.route('dashboard.empresas.edit',$empresa->id).'"><i class="fas fa-pencil-alt"></i></a>';
            }
            if($usuario->hasPermissionTo('Eliminar Empresas') || $usuario->hasRole('Super Admin|Admin')){
                $button .= '<a class="btn btn-danger btn-sm" href="#"   data-toggle="modal" data-target="#empresa-destroy-'.$empresa->id.'"><i class="fas fa-trash"></i></a>';
                $button .= view('empresas::admin.empresas.destroy',['empresa' => $empresa])->render();
            }
            return $button;
        })
        ->addColumn('estado', function($empresa){
            $html = ($empresa->activo) ? '<span class="badge badge-success">Si</span>':'<span class="badge badge-danger">No</span>';

            if($empresa->informe == 1){
                $html .= '&nbsp;<span class="badge badge-info">Standard</span>';
            }else if($empresa->informe == 2){
                $html .= '&nbsp;<span class="badge badge-info">Plus</span>';
            }else{
                $html .= '';
            }

            return $html;
        })
        ->addColumn('dias_restantes', function($empresa){

            $fecha_actual = Carbon::now();

            if($empresa->fecha_habilitacion_servicio){

                $fecha_fin = $empresa->fecha_habilitacion_servicio->addMonth();

                $dias = $fecha_fin->diffInDays($fecha_actual);

                $button = "<span><b>Fecha inicio:</b> {$empresa->fecha_habilitacion_servicio->format('d-m-Y')} </span><br><span><b>Fecha fin:</b> {$fecha_fin->format('d-m-Y')} </span><br><span><b>Días restantes:</b> $dias</span";

            }else{

                $button = "";

            }
            return $button;
        })
        ->rawColumns(['action','estado','dias_restantes'])
        ->make(true);
    }
}
